/***************************************************************************
 * Osama's Lab KS0108 GLCD library; a library that facilitates interfacing KS0108 based GLCDs.
 *
 *   Copyright (C) 2010-2012  Osama's Lab
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Affero General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Affero General Public License for more details.
 *
 *   You should have received a copy of the GNU Affero General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/> 
 **************************************************************************/
#ifndef Glcd_Lib
#define Glcd_Lib
#include "delay.h" 
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "font.h"

//DEBUG
//#define DEBUG_READ 0
//#define DEBUG_GLCD 0
//----------------------
#define E_DELAY 3
#define DATAPORT GPIOC
//#define DATADDR DDRA
//#define DATAPIN PINA
//#define CONTROLPORT PORTB
#define CS1 GPIO_Pin_11
#define CS2 GPIO_Pin_12
#define RS  GPIO_Pin_13
#define RW  GPIO_Pin_9
#define EN  GPIO_Pin_10
//#define CS_ACTIVE_LOW   0   //Define this if your GLCD CS 
                              //is active low (refer to datasheet)
                                                          
//--------Arabic----------
unsigned char prevLet = 193;
static unsigned char stat = 0;
static unsigned char prevX = 0;
static unsigned char prevY = 0;
//------------------------
void trigger()
{
   //EN = 1; //EN high
	GPIO_SetBits(DATAPORT , EN);
   delay_us(E_DELAY);
   //EN = 0; //EN low
	GPIO_ResetBits(DATAPORT , EN);
	
   delay_us(E_DELAY);
}
//----------------------
void glcd_on()
{
    //Activate both chips
    #ifdef CS_ACTIVE_LOW
        //CS1 = 0;
        //CS2 = 0;
		GPIO_ResetBits(DATAPORT , CS1);
		GPIO_ResetBits(DATAPORT , CS2);

    #else    
        //CS1 = 1;         
        //CS2 = 1;
	GPIO_SetBits(DATAPORT , CS1);
	GPIO_SetBits(DATAPORT , CS2);	
    #endif
    //RS = 0;          //RS low --> command
    //RW = 0;          //RW low --> write
		GPIO_ResetBits(DATAPORT , RS);
		GPIO_ResetBits(DATAPORT , RW);
	
    //DATAPORT = 0x3F; //ON command  
		GPIO_ResetBits(DATAPORT , GPIO_Pin_6 | GPIO_Pin_7);
		GPIO_SetBits(DATAPORT , GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 );
		trigger();     
}           
//----------------------
void glcd_off()
{
    //Activate both chips
    #ifdef CS_ACTIVE_LOW
        //CS1 = 0;
        //CS2 = 0;
		GPIO_ResetBits(DATAPORT , CS1);
		GPIO_ResetBits(DATAPORT , CS2);

    #else    
        //CS1 = 1;         
        //CS2 = 1;
	GPIO_SetBits(DATAPORT , CS1);
	GPIO_SetBits(DATAPORT , CS2);	
    #endif

		//RS = 0;          //RS low --> command
    //RW = 0;          //RW low --> write   
		GPIO_ResetBits(DATAPORT , RS);
		GPIO_ResetBits(DATAPORT , RW);
    
		//DATAPORT = 0x3E; //OFF command   
    GPIO_ResetBits(DATAPORT , GPIO_Pin_0 | GPIO_Pin_6 | GPIO_Pin_7);
		GPIO_SetBits(DATAPORT ,GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 );

		trigger();      
}     
//----------------------
void set_start_line(unsigned char line)
{
    //RS = 0;                     //RS low --> command
    //RW = 0;
		GPIO_ResetBits(DATAPORT , RS);
		GPIO_ResetBits(DATAPORT , RW);


		//RW low --> write
    //Activate both chips
    #ifdef CS_ACTIVE_LOW
        //CS1 = 0;
        //CS2 = 0;
		GPIO_ResetBits(DATAPORT , CS1);
		GPIO_ResetBits(DATAPORT , CS2);

    #else    
        //CS1 = 1;         
        //CS2 = 1;
	GPIO_SetBits(DATAPORT , CS1);
	GPIO_SetBits(DATAPORT , CS2);	
    #endif

		//DATAPORT = 0xC0 | line;     //Set Start Line command  
    GPIO_ResetBits(DATAPORT ,GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5);
		GPIO_SetBits(DATAPORT ,GPIO_Pin_6 | GPIO_Pin_7 );


		trigger();     
}
//----------------------
void goto_col(unsigned int x)
{
   unsigned char pattern;
	uint16_t Io_Buffer=0x0000;
   //RS = 0;              //RS low --> command
   //RW = 0;              //RW low --> write   
		GPIO_ResetBits(DATAPORT , RS);
		GPIO_ResetBits(DATAPORT , RW);

	if(x<64)             //left section
   {
      #ifdef CS_ACTIVE_LOW
        //CS1 = 0;          //select chip 1
        //CS2 = 1;          //deselect chip 2
		GPIO_ResetBits(DATAPORT , CS1);
		GPIO_SetBits(DATAPORT , CS2);

		 #else          
        //CS1 = 1;          //select chip 1
        //CS2 = 0;          //deselect chip 2
		GPIO_SetBits(DATAPORT , CS1);
		GPIO_ResetBits(DATAPORT , CS2);
 
     #endif
      pattern = x;      //put column address on data port    
   }
   else                 //right section
   {
      #ifdef CS_ACTIVE_LOW
        //CS2 = 0;
        //CS1 = 1;
		GPIO_ResetBits(DATAPORT , CS2);
		GPIO_SetBits(DATAPORT , CS1);

		 #else
        //CS2 = 1;          //select chip 2
        //CS1 = 0;          //deselct chip 1
		GPIO_SetBits(DATAPORT , CS2);
		GPIO_ResetBits(DATAPORT , CS1);

		 #endif
      pattern = x-64;   //put column address on data port
   }    
   pattern = (pattern | 0x40 ) & 0x7F;  //Command format
   //DATAPORT = pattern; 
	 Io_Buffer=pattern;
	 GPIO_Write(DATAPORT, Io_Buffer);   // write 1 to all 16-bit of Port A

   trigger();     
}   
//----------------------
void goto_row(unsigned int y)
{    
   unsigned char pattern;
	 uint16_t Io_Buffer=0x0000;
	
   //RS = 0;                  //RS low --> command
   //RW = 0;                  //RW low --> write         
		GPIO_ResetBits(DATAPORT , RS);
		GPIO_ResetBits(DATAPORT , RW);

	 pattern = (y | 0xB8 ) & 0xBF; //put row address on data port set command     
   //DATAPORT = pattern;
	 Io_Buffer=pattern;
	 GPIO_Write(DATAPORT, Io_Buffer);   // write 1 to all 16-bit of Port A
	
   trigger();             
}
//----------------------
void goto_xy(unsigned int x,unsigned int y)
{    
    goto_col(x);
    goto_row(y);                                                     
}  
//----------------------
void glcd_write(unsigned char b)
{
		 uint16_t Io_Buffer=0x0000;

   //RS = 1;          //RS high --> data
   //RW = 0;          //RW low --> write     
		GPIO_SetBits(DATAPORT , RS);
		GPIO_ResetBits(DATAPORT , RW);

	//DATAPORT = b;    //put data on data port 
	 Io_Buffer=b;
	 GPIO_Write(DATAPORT, Io_Buffer);   // write 1 to all 16-bit of Port A
	
   delay_us(1);   
   trigger();
}
//------------------------
void glcd_clrln(unsigned char ln)
{
    int i;
    goto_xy(0,ln);      //At start of line of left side
    goto_xy(64,ln);     //At start of line of right side (Problem)
    #ifdef CS_ACTIVE_LOW
        //CS1 = 0;
		GPIO_ResetBits(DATAPORT , CS1);
    #else
        //CS1 = 1;
		GPIO_SetBits(DATAPORT , CS1);

    #endif    
    for(i=0;i<65;i++)
        glcd_write(0);
}
//-------------------------
void glcd_clear()
{
   int i;
   for(i=0;i<8;i++)
      glcd_clrln(i);
}
//-----------------------
unsigned char is_busy()
{
    unsigned char status = 0;        //Read data here
    
    //EN = 0;                 //Low Enable
		GPIO_ResetBits(DATAPORT , EN);
	
    delay_us(1);            //tf
    //RW = 1;                 //Read
    //RS = 0;                 //Status    
		GPIO_SetBits(DATAPORT , RW);
		GPIO_ResetBits(DATAPORT , RS);
	
    delay_us(1);            //tasu
    //EN = 1;                 //High Enable
		GPIO_SetBits(DATAPORT , EN);
	
    delay_us(5);            //tr + max(td,twh)->twh
    
    //Dummy read
    //EN = 0;                 //Low Enable
		GPIO_ResetBits(DATAPORT , EN);
	
    delay_us(5);            //tf + twl + chineese error    
    
    //EN = 1;                 //High Enable
		GPIO_SetBits(DATAPORT , EN);
		
    delay_us(1);            //tr + td        
                                  
    //status = DATAPIN;    //Input data
		status = GPIO_ReadInputData	(	DATAPORT ); 	
    //EN = 0;                 //Low Enable
		GPIO_ResetBits(DATAPORT , EN);
		
    delay_us(1);            //tdhr
    #ifdef DEBUG_READ
        printf("S:%x\n\r",status);
    #endif
    return (status & 0x80);           
}
//-----------------------   
unsigned char glcd_read(unsigned char column)
{
    unsigned char read_data = 0; //Read data here
    //DATADDR = 0x00;     //Input
		GPIO_Input();
    //while(is_busy());
    //RW = 1;             //Read
    //RS = 1;             //Data
		GPIO_SetBits(DATAPORT , RW);
		GPIO_SetBits(DATAPORT , RS);
	
    #ifdef CS_ACTIVE_LOW
        //CS1 = (column>63);
	if(column>63)
		GPIO_SetBits(DATAPORT , CS1);
	else
		GPIO_ResetBits(DATAPORT , CS1);
	
    #else         
        //CS1 = (column<64);  //Enable/Disable CS1
	if(column<64)
		GPIO_SetBits(DATAPORT , CS1);
	else
		GPIO_ResetBits(DATAPORT , CS1);
	
	
    #endif
    //CS2 = !CS1;         //Disable/Enable CS2
		if ((GPIO_ReadInputDataBit(DATAPORT, CS2)) == 0)
		GPIO_SetBits(DATAPORT , CS2);
		else if ((GPIO_ReadInputDataBit(DATAPORT, CS2)) == 1)
		GPIO_ResetBits(DATAPORT , CS2);
	
		
    delay_us(1);        //tasu
    //EN = 1;             //Latch RAM data into ouput register
		GPIO_SetBits(DATAPORT , EN);
	
    delay_us(1);        //twl + tf
    
    //Dummy read
    //while(is_busy());
    //EN = 0;             //Low Enable
		GPIO_ResetBits(DATAPORT , EN);
		
    delay_us(20);       //tf + twl + chineese error    
    
    //EN = 1;             //latch data from output register to data bus
		GPIO_SetBits(DATAPORT , EN);
		
    delay_us(1);        //tr + td(twh)       
                                  
    //read_data = DATAPIN;    //Input data
		read_data = GPIO_ReadInputData	(	DATAPORT ); 	
    
    //EN = 0;             //Low Enable to remove data from the bus 
		GPIO_ResetBits(DATAPORT , EN);
		
    delay_us(1);        //tdhr
    #ifdef DEBUG_READ
        printf("R:%x\n\r",read_data);
    #endif
    //DATADDR = 0xFF;     //Output again
		GPIO_Output();
    return read_data;      
}
//-----------------------
/*unsigned char get_point(unsigned int x,unsigned int y)
{
    unsigned char data;
    goto_xy(x,((int)(y/8)));
    data = glcd_read();        
    return data;
}  */
//----------------------
void point_at(unsigned int x,unsigned int y,unsigned char color)
{
    unsigned char pattern;
    goto_xy(x,(int)(y/8));  
    switch (color)
    {               
        case 0:         //Light spot                        
            pattern = ~(1<<(y%8)) & glcd_read(x);                  
        break;    
        case 1:         //Dark spot            
            pattern = (1<<(y%8)) | glcd_read(x);
            #ifdef DEBUG_GLCD
                delay_ms(30);/////////////////////////////////////////
            #endif
        break;
    }           
    goto_xy(x,(int)(y/8));                                
    glcd_write(pattern);
}
//-----------------------
void h_line(unsigned int x,unsigned int y,unsigned char l,unsigned char s,unsigned char c)
{
    int i;
    for(i=x; i<(l+x); i += (unsigned char)(s+1))
        point_at(i,y,c);
}
//-----------------------
void v_line(unsigned int x,unsigned int y,signed int l,unsigned char s,unsigned char c)
{
    unsigned int i;
    for(i=y; i<(y+l); i += (unsigned char)(s+1))
        point_at(x,i,c);
}
//-----------------------
void line(unsigned int x1,unsigned int y1,
          unsigned int x2,unsigned int y2,
          unsigned char s,unsigned char c)
{
    unsigned char i;
    unsigned char y01;
    
    unsigned char temp;
    
    float a;
    float b;
    
    unsigned char y00;
    unsigned char y010;
    
    if(x1==x2) 
    {    
        v_line(x1,min(y1,y2),abs(y2-y1)+1,s,c);
    }
    else if(y1==y2)
    {
        h_line(min(x1,x2),y1,abs(x2-x1)+1,s,c);
    }
    else    
    {
        if(x1>x2)
        {
            temp = x1;
            x1 = x2;
            x2 = temp;
            temp = y1;
            y1 = y2;
            y2 = temp;
        }
        
        a = (float)(signed)(y2-y1)/(x2-x1);
        b = y1 - a*x1;
        y00 = a*x1 + b;
        y010 = y00;    
                   
        for(i=(x1+1); i<=x2; i ++)
        {
            y01 = a*i + b + 0.5; //+ 0.5 to approximate to the nearest integer
                
            if( (signed char)(y01 - y010) > 1 )
            {
                //-------Subject to error
                v_line((i-1),(unsigned char)(y010+1),(y01-y010-1),s,c);
            }
            else if( (signed char)(y010 - y01) > 1 )
            {
                //-------Subject to error
                v_line((i-1),(unsigned char)(y01+1),(y010-y01-1),s,c);
            } 
                       
            if((i==x2) && (y00 == y01)) h_line(x1,y01,(x2-x1),0,c);                                  
            y010 = y01;       
            if( y00 ==  y01) continue;
            h_line(x1,y00,(i-x1),0,c);
            x1 = i;
            y00 = y01;
        }
        point_at(x2,y2,c); 
    }          
}
//-----------------------
void rectangle(unsigned int x1,unsigned int y1,
               unsigned int x2,unsigned int y2,
               unsigned char s,unsigned char c)
{
    h_line(x1,y1,(x2-x1),s,c);
    h_line(x1,y2,(x2-x1),s,c);
    v_line(x1,y1,(y2-y1),s,c);
    v_line(x2,y1,(y2-y1+1),s,c);
}
//-----------------------
void cuboid(unsigned int x11,unsigned int y11,
            unsigned int x12,unsigned int y12,
            unsigned int x21,unsigned int y21,
            unsigned int x22,unsigned int y22,
            unsigned char s,unsigned char c)
{
    rectangle(x11,y11,x12,y12,s,c);
    rectangle(x21,y21,x22,y22,s,c);
    line(x11,y11,x21,y21,s,c);
    line(x12,y11,x22,y21,s,c);
    line(x11,y12,x21,y22,s,c);
    line(x12,y12,x22,y22,s,c);
}
//-----------------------
void h_parallelogram(unsigned int x1,unsigned int y1,
                     unsigned int x2,unsigned int y2,
                     unsigned char l,unsigned char s,unsigned char c)
{
    h_line(x1,y1,l,s,c);
    h_line((x2-l+1),y2,l,s,c);
    line(x1,y1,(x2-l+1),y2,s,c);
    line((x1+l-1),y1,x2,y2,s,c);
}
//-----------------------
void v_parallelogram(unsigned int x1,unsigned int y1,
                     unsigned int x2,unsigned int y2,
                     unsigned char l,unsigned char s,unsigned char c)
{
    v_line(x1,y1,l,s,c);
    v_line(x2,(y2-l+1),l,s,c);
    line(x1,y1,x2,(y2-l+1),s,c);
    line(x1,(y1+l-1),x2,y2,s,c);
}
//-----------------------
void h_parallelepiped(unsigned int x11,unsigned int y11,
                      unsigned int x12,unsigned int y12,unsigned char l1,
                      unsigned int x21,unsigned int y21,
                      unsigned int x22,unsigned int y22,unsigned char l2,
                      unsigned char s,unsigned char c)
{
    h_parallelogram(x11,y11,x12,y12,l1,s,c);
    h_parallelogram(x21,y21,x22,y22,l2,s,c);
    line(x11,y11,x21,y21,s,c);
    line(x12,y12,x22,y22,s,c);
    line((x11+l1-1),y11,(x21+l2-1),y21,s,c);
    line((x12-l1+1),y12,(x22-l2+1),y22,s,c);
}
//-----------------------
void v_parallelepiped(unsigned int x11,unsigned int y11,
                      unsigned int x12,unsigned int y12,unsigned char l1,
                      unsigned int x21,unsigned int y21,
                      unsigned int x22,unsigned int y22,unsigned char l2,
                      unsigned char s,unsigned char c)
{
    v_parallelogram(x11,y11,x12,y12,l1,s,c);
    v_parallelogram(x21,y21,x22,y22,l2,s,c);
    line(x11,y11,x21,y21,s,c);
    line(x12,y12,x22,y22,s,c);
    line(x11,(y11+l1-1),x21,(y21+l2-1),s,c);
    line(x12,(y12-l1+1),x22,(y22-l2+1),s,c);
}
//-----------------------
void circle(unsigned int x0,unsigned int y0,
            unsigned int r,unsigned char s,unsigned char c)
{
    unsigned char i,y,y00;
    y00 = r;            //Point (0,r) is the 1st point
    for(i=0; i<r; i++)
    {
        y = sqrt((int)r*r - (int)i*i);
        point_at((x0+i),(y0+y),c);
        point_at((x0+i),(y0-y),c);
        point_at((x0-i),(y0+y),c);
        point_at((x0-i),(y0-y),c);        
        if(abs(y00-y)>1)
        {
            v_line(x0+(i-1),y0+min(y00,y)+1,abs(y00-y),s,c);
            v_line(x0+(i-1),y0-max(y00,y),abs(y00-y),s,c);
            v_line(x0-(i-1),y0+min(y00,y)+1,abs(y00-y),s,c);
            v_line(x0-(i-1),y0-max(y00,y),abs(y00-y),s,c);
        }
        y00 = y;   
    }
    v_line(x0+(i-1) ,y0,y ,s,c);
    v_line(x0+(i-1) ,y0-y ,y,s,c);
    v_line(x0-(i-1) ,y0,y ,s,c);
    v_line(x0-(i-1) ,y0-y ,y,s,c);    
}
//-----------------------
//void ellipse(unsigned int x0,unsigned int y0,
//             unsigned int a,unsigned int b,
//             unsigned char s,unsigned char c)
//{
//    unsigned char i,y,y00;
//    y00 = 1;
//    for(i=0; i < a; i++)
//    {
//        y = isqrt((int)a*a - (int)i*i);
//        point_at((x0+i)*(a+x0)/a,(y0+y)*(b+y0)/b,c);
//        point_at((x0+i)*(a+x0)/a,(y0-y)*(b+y0)/b,c);
//        point_at((x0-i)*(a+x0)/a,(y0+y)*(b+y0)/b,c);
//        point_at((x0-i)*(a+x0)/a,(y0-y)*(b+y0)/b,c);        
//        /*if(abs(y00-y)>1)
//        {
//            v_line(x0+(i-1),y0+min(y00,y)+1,abs(y00-y),s,c);
//            v_line(x0+(i-1),y0-max(y00,y),abs(y00-y),s,c);
//            v_line(x0-(i-1),y0+min(y00,y)+1,abs(y00-y),s,c);
//            v_line(x0-(i-1),y0-max(y00,y),abs(y00-y),s,c);
//            
//            printf("y=%u y00=%u\n\r",y,y00);
//        }*/
//        y00 = y;   
//    }
//    /*v_line(x0+(i-1) ,y0,y ,s,c);
//    v_line(x0+(i-1) ,y0-y ,y,s,c);
//    v_line(x0-(i-1) ,y0,y ,s,c);
//    v_line(x0-(i-1) ,y0-y ,y,s,c);*/    
//}  
//------------Arabic/English Writing-------
void putIt(int c,int x,int y)
{
   unsigned char i;
  goto_col(x);
  goto_row(y);
  for(i=0;i<8;i++)
  {
     glcd_write(font[(8*c)+i]);
  }
}
//--------------------------
void enlarge(char *large,unsigned char c,unsigned char size)
{
	unsigned char i,j,temp;
	unsigned char k = 0;
	for(i=0;i<size;i++)
	{
		for(j=0;j<8;j++)
		{
			large[i] >>= 1;
			temp = c & 0x01;
			if(temp)
				large[i] |= 0x80;
			if(++k == size)
			{
				c >>= 1;
				k = 0;
			}
		}
	}
}
//--------------------------
void putItSz(int c,int x,int y,unsigned char sz)
{
  unsigned char i,j,k;
  //char large[8][sz];  
  char large[8][8];
  
  goto_col(x);
  goto_row(y);
  for(i=0;i<8;i++)
  {                                     
	  enlarge(large[i],font[(8*c)+i],sz);		//c here;GCC (c-1)
  }
  for(j=0;j<sz;j++)
  {
	  for(i=0;i<8;i++)
	  {
		  for(k=0;k<sz;k++)
		  {
			  if(x+k+(sz*i) == 64) goto_xy(64,y);		//Problem 1: Forgot to change 3 to sz
			  glcd_write(large[i][j]);
		  }
	  }
	  goto_xy(x,++y);
  }
}
//--------------------------
void glcd_putchar(unsigned char c,int x,int y,unsigned char l,unsigned char sz)
{
    if(l == 1)
    {       
        switch(c)
    	{
    	case 129:
    		c = 250;
    		break;
    	case 144:
    		c = 251;
    		break;
    	case 152:
    		c = 252;
    		break;
    	case 142:
    		c = 253;
    		break;
    	case 141:
    		c = 254;
    		break;
    	}
        if((c >= 193) && (prevLet >= 193) && (map[prevLet-193][5]) && (map[c-193][4]))
        {
            putIt(map[prevLet-193][stat+1],prevX,prevY);
            stat = 2;
        }else stat = 0;

        if(c >= 193) putItSz(map[c-193][stat],x,y,sz);
        else putItSz(c,x,y,sz);

        prevLet = c;
        prevX = x;
        prevY = y;

    }else putItSz(c,x,y,sz);
}
//---------------------------
void glcd_puts( char *c,int x,int y,unsigned char l,unsigned char sz,signed char space)
{
   char i = 0;
	 unsigned char Display_Buffer[sizeof(c)],counter;
   char special = 0;
   while((i<strlen(c)) && l==0)
   {
      glcd_putchar(*(c+i),x,y,0,sz);
      x += (8+space)*sz;
      if(x>128-8*sz)
      {
         x=0;
         y += sz;
      }
      i++;
   }
	 for(counter=0;counter<sizeof(c);counter++)
	 Display_Buffer[counter]=c[counter];
	 
   while((i<strlen(c)) && l==1)
   {
      if((*(Display_Buffer+i) == 225) && (*(Display_Buffer+i+1) == 199))
        special = 249;
      else if((*(Display_Buffer+i) == 225) && (*(Display_Buffer+i+1) == 195))
        special = 231;
      else if((*(Display_Buffer+i) == 225) && (*(Display_Buffer+i+1) == 194))
        special = 232;
      else if((*(Display_Buffer+i) == 225) && (*(Display_Buffer+i+1) == 197))
        special = 233;
      if(special)
      {
        glcd_putchar(special,x-8*sz,y,1,sz);
        i+=2;
        x -= 8*sz;
        special = 0;
      }      
      else
      {
		  glcd_putchar(*(c+i),x-8*sz,y,l,sz);
		  if(*(c+i) == 32)						//If space (i.e. new word)
		  {
			  x -= (8+space)*sz;				//Space between words
		  }
		  else
		  {
			  x -= 8*sz;
		  }
		  i++;
      } 
      if(x < 8*sz+1)
      {
         x=128-8*sz;
         y += sz;
      }
   }
   prevLet = 193;
}
//------------BMP Display--------------------
void bmp_disp( unsigned char *bmp,unsigned int x1,unsigned int y1,
                              unsigned int x2,unsigned int y2)
{
    unsigned int i,j;    
    for(i=y1;i<=y2;i++)
    {
        for(j=x1;j<=x2;j++)
        {                       
           goto_xy(j,i);                              
           glcd_write(bmp[(128*i)+j]);
        }                                              
    }
}
 

#endif
